<?php
/**
 * Vista API Requests
 * 
 * This file is included in inc.lib.html.php. It serves
 * as the Vista API connection layer.
 * 
 * @package wwm_lib_php
 * @version 0.1
 * @author Rob Fonseca <rob@westworldmedia.com>
 * @copyright West World Media ©2014
 */
/*********************
Vista API Requests
Written by Rob Fonseca
West World Media ©2014
**********************/

/*
* REQUIRED. MUST CHANGE SETUP PER CUSTOMER
* Assigns default connection info to an array
*
* @param $vista_url
* @param $cinema_id
*
* return two dimmensional array $vista
*/
function vista_connection_info($vista_url, $cinema_id) {
	
	//setup array
	$vista = array();
	
	//fill in default variables for customer
	$vista["loyalty_server"] = "http://$vista_url/WSVistaWebClient/LoyaltyService.asmx?WSDL";
	$vista["data_server"] = "http://$vista_url/WSVistaWebClient/DataService.asmx?WSDL";
	$vista["time"] = date("YmdHis");
	$vista["cinema_id"] = $cinema_id;
	$vista["loyalty_server"] = "";
	$vista["logging_level"] = "";
	$vista["club_id"] = "";
	
	//retunr the array
	return $vista;
}

/*
* Connects to the Vista loyalty API
*
* @param $type
* @param $vista_url
* @param $cinema_id
*
* return SOAP connection object
*/
function vista_connect($type, $vista_url, $cinema_id) {
	
	//get default connection variables
	$vista = vista_connection_info($vista_url, $cinema_id);
	
	//which service are we connecting to?
	if ($type == "loyalty") {
		$server = $vista["loyalty_server"];
	} elseif ($type == "data") {
		$server = $vista["data_server"];
	}
	
	//create the SOAP client connection
	$soapClient = new SoapClient($server, array ('exceptions' => 1,'trace' => 0));
	return $soapClient;
}

/*
* Sends a request packet to Vista LoyaltyService API
*
* @param $soapClient
* @param $packet
*
* return XML string $response
*/
function vista_send_loyalty_request($soapClient, $packet) {
	
	//get default connection variables
	$parameters = array('XMLMsgIn' => $packet);
	$response = $soapClient->__soapCall('InvokeLoyalty', array($parameters));
	$response = str_ireplace('UTF-16', 'UTF-8', $response->InvokeLoyaltyResult);
	
	//return the response XML
	return $response;
}

/*
* Sends a request packet to Vista DataService API
*
* @param $soapClient
* @param $packet
* @param $function
*
* return XML string on success, "ERROR" string on failure
*/
function vista_send_data_request($soapClient, $packet, $function) {
	
	//request data
	$response = $soapClient->__soapCall($function, array($packet));
	
	//return the response object
	return $response;
}

/*
* Gets a loyalty card balance
*
* @param $soapClient
* @param $vista_url
* @param $cinema_id
* @param $card_number
*
* return XML string $response
*/
function vista_loyalty_balance($soapClient, $vista_url, $cinema_id, $card_number) {
	
	//get default connection variables
	$vista = vista_connection_info($vista_url, $cinema_id);
	
	//build the request packet
	$packet = '<?xml version="1.0"?>
					<VistaLoyaltyMsg>
					       <!--Vista Entertainment Solutions - All Rights Reserved-->
					       <MsgID/>
					       <MsgType>REQBAL</MsgType>
					       <MsgTime>' . $vista["time"] . '</MsgTime>
					       <MsgResponseReq>Y</MsgResponseReq>
					       <DataItem DataItemType="MsgParamList">
					             <Param Name="CinemaID">' . $vista["cinema_id"] . '</Param>
					             <Param Name="LoyaltyServer">' . $vista["loyalty_server"] . '</Param>
					             <Param Name="LoggingLevel">' . $vista["logging_level"] . '</Param>
					             <Param Name="CardNumber">' . $card_number . '</Param>
					       </DataItem>
					</VistaLoyaltyMsg>';
	
	//send the request and return the XML response string
	return vista_send_loyalty_request($soapClient, $packet);
}

/*
* Gets a gift card balance
*
* @param $soapClient
* @param $vista_url
* @param $cinema_id
* @param $card_number
*
* return XML string $response
*/
function vista_gift_card_balance($soapClient, $vista_url, $cinema_id, $card_number) {
	
	//get default connection variables
	$vista = vista_connection_info($vista_url, $cinema_id);
	
	//build the packet array	
	$packet = array("UseGiftCardBalanceMethod" => "CardNum", "CardNo" => $card_number, "CinemaID" => $vista["cinema_id"]);

	//return the data and parse the response
	$response = vista_send_data_request($soapClient, $packet, 'GiftCardBalanceRequest');
	$xml = parse_response($response);
	
	//return the balance if valid XML
	if ($xml != "ERROR" && stripos($xml, "failed") == 0) {
		return get_xml_field($xml, 'CardBalance');
	} else {
		return "ERROR";
	}
}

/*
* Gets showtimes for a particular theatre
*
* @param $soapClient
* @param $vista_url
* @param $cinema_id
* @param $cinema_id
*
* return XML output
*/
function vista_get_showtimes_xml($soapClient, $vista_url, $cinema_id, $link_root) {
	
	//kill if ID is blank or not numeric
	if (!is_numeric($cinema_id) || $cinema_id == "")
		die();
	
	//movie list
	$packet = array("OptionalCinemaId" => $cinema_id, "OptionalOrderByOperator" => "", "OptionalBizStartTimeOfDay" => "", 
					"OptionalIncludeGiftStores" => "");
	$xml = vista_send_data_request($soapClient, $packet, "GetMovieList");
	$xml = '<?xml version="1.0" encoding="UTF-8"?>' . $xml->{DatasetXML};
	//echo $xml;
	
	//create XML object
	$xml = new SimpleXMLElement($xml);
	
	//count the movies
	$movie_count = count($xml->Table);
	
	//start the XML
	header('Content-Type: text/xml');
	$output =  "<?xml version='1.0' encoding='UTF-8'?>
					<vistashowtimes>
					<theatreid>$cinema_id</theatreid>";
	
	//loop through movies and get showtimes
	foreach($xml->Table as $m) {
		
		//movie ID and name
		$movie_id = $m->Movie_strID;
		$name =  $m->Movie_strName;
		$rating = $m->Movie_strRating;
		
		//showtimes
		$packet = array("CinemaId" => $cinema_id, "MovieId" => $movie_id, "OptionalBizStartTimeOfDay" => "", 
						"OptionalBizStartHourOfDay" => "", "OptionalSessionDisplayCutOff" => "");
		$show_xml = vista_send_data_request($soapClient, $packet, "GetSessionList");
		$show_xml = '<?xml version="1.0" encoding="UTF-8"?>' . $show_xml->{DatasetXML};
		//echo $show_xml;
		$show_xml = new SimpleXMLElement($show_xml);
		
		//start movie node
		$output .=  "<filmtitle>
					<id>$movie_id</id>
					<name>$name</name>
					<shortname>$name</shortname>
					<rating>$rating</rating>
					<length>0</length>";
		
		foreach($show_xml->Table as $s) {
			
			//get showtime info
			$code = $s->Movie_strID;
			$time = substr($s->Session_dtmDate_Time, strpos($s->Session_dtmDate_Time, "T")+1, 5);
			$orig_date = substr($s->Session_dtmBizDate, 0, 10);
			$date = date("mdY", strtotime($orig_date));
			$session_id = $s->Session_strID;
			$link = "$link_root?cinemacode=$cinema_id&amp;txtSessionId=$session_id";
			
			//Midnight - 2:59AM gets moved to the previous day!
			$hour = substr($time,0,2);
			if ($hour == "00" || $hour == "01" || $hour == "02") {
				$date = date("mdY", strtotime("$orig_date -1day"));
			} 
 
			
			//remove text from screen data
			$screen = preg_replace("/[^0-9,]/", "", $s->Screen_strName);
			
			//build comment string
			$comments = $s->Session_strAttributes;
			$comments = str_ireplace("MX", "AVX MAX", $comments);
			
			//add DBOX? Only if not in comment string already
			if (strpos($name, "DBOX") > 0 && strpos($comments, "DBOX") == 0) {
				$comments = "DBOX;" . $comments;
			}
					
			//echo out fields
			$output .= "<show>
						<screen>$screen</screen>
						<date>$date</date>
						<time>$time</time>
						<comments>$comments</comments>
						<soldout>0</soldout>
						<soldoutGen>0</soldoutGen>
						<reserved>0</reserved>
						<salelink>$link</salelink>
					  </show>";
		}
		
		//end movie title
		$output .= "</filmtitle>";
		
	}
	
	//close out XML
	$output .= "</vistashowtimes>";
	
	//retirn the XML output
	$xml = NULL;
	$show_xml = NULL;
	return $output;
}

/*
* Gets alist if cinemas
*
* @param $soapClient
*
* return XML string $response
*/
function vista_get_cinema_list($soapClient) {
	
	//get default connection variables
	$response = $soapClient->__soapCall('GetCinemaListAll', array());
	return $response->{"DatasetXML"};
}

/*
* Activates a Vitsa Loyalty Membership
*
* @param $soapClient
* @param $vista_url
* @param $cinema_id
* @param $card_number
* @param $phone
* @param $first_name
* @param $last_name
* @param $address
* @param $emaiL_address
*
* return XML string $response
*/
function vista_loyalty_create_member($soapClient, $vista_url, $cinema_id, $card_number, $phone, $first_name, $last_name, $address, $email_address) {
	
	//get default connection variables
	$vista = vista_connection_info($vista_url, $cinema_id);
	
	//build the request packet
	$packet = '<?xml version="1.0"?>
					<VistaLoyaltyMsg>
					       <!--Vista Entertainment Solutions - All Rights Reserved-->
					       <MsgID/>
					       <MsgType>ACTIVATE</MsgType>
					       <MsgTime>' . $vista["time"] . '</MsgTime>
					       <MsgResponseReq>Y</MsgResponseReq>
					       <DataItem DataItemType="MsgParamList">
					             <Param Name="CinemaID">' . $vista["cinema_id"] . '</Param>
					             <Param Name="LoyaltyServer">' . $vista["loyalty_server"] . '</Param>
					             <Param Name="LoggingLevel">' . $vista["logging_level"] . '</Param>';
		
	//add the card number if they entered one	             
	if ($card_number != "") {
		$packet .= '<Param Name="CardNumber">' . $card_number . '</Param>';										
	}
	
	//finish the packet
	$packet .= '<Param Name="MemberHomePhone">' . $phone . '</Param>
                <Param Name="MemberFirstName">' . $first_name . '</Param>
                <Param Name="MemberLastName">' . $last_name . '</Param>
                <Param Name="MemberAddress1">' . $address . '</Param>
                <Param Name="ClubID">' . $vista["club_id"]  . '</Param>
                <Param Name="MemberEmail">' . $email_address . '</Param>
			</DataItem>
		</VistaLoyaltyMsg>';
	
	//send the request and return the XML response string
	return vista_send_loyalty_request($soapClient, $packet);
}

/*
* Activate a Vista loyalty membership
*
* @param $soapClient
* @param $vista_url
* @param $cinema_id
* @param $card_number
*
* return XML string $response
*/
function vista_loyalty_activate_member($soapClient, $vista_url, $cinema_id, $member_id) {
	
	//get default connection variables
	$vista = vista_connection_info($vista_url, $cinema_id);
	
	//build the request packet
	$packet = '<?xml version="1.0"?>
					<VistaLoyaltyMsg>
					       <!--Vista Entertainment Solutions - All Rights Reserved-->
					       <MsgID/>
					       <MsgType>ACTIVATE</MsgType>
					       <MsgTime>' . $vista["time"] . '</MsgTime>
					       <MsgResponseReq>Y</MsgResponseReq>
					       <DataItem DataItemType="MsgParamList">
					             <Param Name="CinemaID">' . $vista["cinema_id"] . '</Param>
					             <Param Name="LoyaltyServer">' . $vista["loyalty_server"] . '</Param>
					             <Param Name="LoggingLevel">' . $vista["logging_level"] . '</Param>
					             <Param Name="Activated">Y</Param>
								 <Param Name="MembershipID">' . $member_id . '</Param>
					       </DataItem>
				    </VistaLoyaltyMsg>';
	
	//send the request and return the XML response string
	return vista_send_loyalty_request($soapClient, $packet);
}

/*
* Updates a Vista Loyalty Membership
*
* @param $soapClient
* @param $vista_url
* @param $cinema_id
* @param $card_number
* @param $phone
* @param $first_name
* @param $last_name
* @param $address
* @param $email_address
* @param $member_id
*
* return XML string $response
*/
function vista_loyalty_update_member($soapClient, $vista_url, $cinema_id, $card_number, $phone, $first_name, $last_name, $address, $email_address, $member_id) {
	
	//get default connection variables
	$vista = vista_connection_info($vista_url, $cinema_id);
	
	//build the request packet
	$packet = '<?xml version="1.0"?>
					<VistaLoyaltyMsg>
					       <!--Vista Entertainment Solutions - All Rights Reserved-->
					       <MsgID/>
					       <MsgType>ACTIVATE</MsgType>
					       <MsgTime>' . $vista["time"] . '</MsgTime>
					       <MsgResponseReq>Y</MsgResponseReq>
					       <DataItem DataItemType="MsgParamList">
					             <Param Name="CinemaID">' . $vista["cinema_id"] . '</Param>
					             <Param Name="LoyaltyServer">' . $vista["loyalty_server"] . '</Param>
					             <Param Name="LoggingLevel">' . $vista["logging_level"] . '</Param>
					             <Param Name="MembershipID">' . $member_id . '</Param>
								 <Param Name="MemberHomePhone">' . $phone . '</Param>
				                 <Param Name="MemberFirstName">' . $first_name . '</Param>
				                 <Param Name="MemberLastName">' . $last_name . '</Param>
				                 <Param Name="MemberAddress1">' . $address . '</Param>
				                 <Param Name="ClubID">' . $vista["club_id"] . '</Param>
				                 <Param Name="MemberEmail">' . $email_address . '</Param>
							</DataItem>
						</VistaLoyaltyMsg>';
	
	//send the request and return the XML response string
	return vista_send_loyalty_request($soapClient, $packet);
}

/*
* Find a Vista loyalty membership by email address
*
* @param $soapClient
* @param $vista_url
* @param $cinema_id
* @param $email_address
*
* return XML string $response
*/
function vista_loyalty_find_member($soapClient, $vista_url, $cinema_id, $email_address) {
	
	//get default connection variables
	$vista = vista_connection_info($vista_url, $cinema_id);
	
	//build the request packet
	$packet = '<?xml version="1.0"?>
					<VistaLoyaltyMsg>
					       <!--Vista Entertainment Solutions - All Rights Reserved-->
					       <MsgID/>
					       <MsgType>MBRSEARCH</MsgType>
					       <MsgTime>' . $vista["time"] . '</MsgTime>
					       <MsgResponseReq>Y</MsgResponseReq>
					       <DataItem DataItemType="MsgParamList">
					              <Param Name="CinemaID">' . $vista["cinema_id"] . '</Param>
					             <Param Name="LoyaltyServer">' . $vista["loyalty_server"] . '</Param>
					             <Param Name="LoggingLevel">' . $vista["logging_level"] . '</Param>
					             <Param Name="MemberEmail">' . $email_address . '</Param>
					       </DataItem>
					</VistaLoyaltyMsg>';
	
	//send the request and return the XML response string
	return vista_send_loyalty_request($soapClient, $packet);
}

/*
* Delete a Vista loyalty membership
*
* @param $soapClient
* @param $vista_url
* @param $cinema_id
* @param $card_number
* @param $member_id
* @param $user_id
*
* return XML string $response
*/
function vista_loyalty_delete_member($soapClient, $vista_url, $cinema_id, $card_number, $member_id, $user_id) {
	
	//get default connection variables
	$vista = vista_connection_info($vista_url, $cinema_id);
	
	//build the request packet
	$packet = '<?xml version="1.0"?>
					<VistaLoyaltyMsg>
					       <!--Vista Entertainment Solutions - All Rights Reserved-->
					       <MsgID/>
					       <MsgType>MBRSEARCH</MsgType>
					       <MsgTime>' . $vista["time"] . '</MsgTime>
					       <MsgResponseReq>Y</MsgResponseReq>
					       <DataItem DataItemType="MsgParamList">
					              <Param Name="CinemaID">' . $vista["cinema_id"] . '</Param>
					             <Param Name="LoyaltyServer">' . $vista["loyalty_server"] . '</Param>
					             <Param Name="LoggingLevel">' . $vista["logging_level"] . '</Param>
					             <Param Name="MemberEmail">' . $email_address . '</Param>
					       </DataItem>
					</VistaLoyaltyMsg>';
		
	//send the request and return the XML response string
	return vista_send_loyalty_request($soapClient, $packet);
}

/*
 * Parse a SOAP response object from Vista DataService API
 *
 * @param $xml
 * @param $field
 *
 * return string $value
 */
function parse_response($obj) {
	
	//set up start and end text
	$result = $obj->{"Result"};

	//Return XML if result is OK, ERROR if not
	if ($result == "OK") {
		$xml = $obj->{"DatasetXML"};
		return $xml;
	} else {
		return "ERROR";
	}
}

/*
 * Get a response field from Vista API
 *
 * @param $xml
 * @param $field
 *
 * return string $value
 */
function get_loyalty_field($xml, $field) {
	
	//set up start and end text
	$start_field = '<Param Name="' . $field . '">';
	$end_field = '</Param>';
	
	//determine position of start and end text
	$start = strpos($xml, $start_field) + strlen($start_field);
	$end = strpos($xml, $end_field, $start);
	
	//return the value of the parameter field
	return substr($xml, $start, $end - $start);
}

/*
 * Get certain field from response XML
 *
 * @param $xml
 * @param $field
 *
 * return string $value
 */
function get_xml_field($xml, $field) {
	
	//create a XML object if we have a XML response
	if (strpos($xml, ">") > 0) {
		$xml = simplexml_load_string($xml);
	}
	
	//Card Balance
	if ($field == "CardBalance") {
		return "$" . number_format(($xml->Table->CardBalance / 100), "2");
	}
}

?>